part of tecfy_core_client_package;

class EntityListSearchComponent extends StatefulWidget {
  final Entity entity;
  final String url;
  final void Function(dynamic) onClick;
  // final String filterField;
  // final dynamic filterValue;
  // final bool isFilter;
  final dynamic urlFilterData;
  final Map<String,dynamic> data;
  final PropertyDropdown property;

  EntityListSearchComponent({this.property,this.onClick,this.entity,this.data,this.url,this.urlFilterData}): assert(entity != null);
  @override
  _EntityListSearchComponentState createState() => _EntityListSearchComponentState();
}

class _EntityListSearchComponentState extends State<EntityListSearchComponent> {
  
  List<dynamic> data;
  List<dynamic> dataFilterd;
  Entity entity;
  ScrollController scrollController;
  int pageIndex=0;
  @override
  void initState() {

    scrollController = ScrollController();
    scrollController.addListener(_scrollListner);

    //print(widget.entity);
    entity = widget.entity;
    //print(entity);
    getList().then((result){
      if(result.success){
        data = result.data.where((d)=> !widget.property.isFilter || widget.data[widget.property.filterValueField] == d[widget.property.filterField]).toList();
        setState(() {});
      }else result.showMessage(context);
    });
    super.initState();
  }

  @override dispose(){
    scrollController?.removeListener(_scrollListner);
    scrollController?.dispose();
    super.dispose();
  }

  _scrollListner(){

    if(scrollController.position.extentAfter < 100){
      pageIndex++;
      EntityService().getData(widget.entity,forceFromServer:true,pageIndex: pageIndex,activeOnly:true).then((result){
        data.addAll(result.data);
        setState(() {});
      });
    }
  }

  Future<OperationResult<List<dynamic>>> getList({forceFromServer:false})async{
    try{
    print('+++++++++++ ${widget.property.filterField} , ${widget.property.filterValueField},  ${widget.data[widget.property.filterValueField]}');
    if(widget.url == null || widget.url.length == 0){

      if(!widget.property.isFilter || widget.property.filterValueField == null || widget.property.filterValueField.toString().length ==0){
        print('no filter');
        return await EntityService().getData(widget.entity,forceFromServer:forceFromServer,activeOnly:true);
      }
      else{
        if(widget.property.filterField == null || widget.property.filterField.toString().length == 0){
          if(widget.property.filterValueField.toString().contains('.')){
            
            var parentPropName = widget.property.filterValueField.split('.')[0];
            var childPropName =widget.property.filterValueField.split('.')[1];

            var parentData = widget.data[widget.property.lowerFirst(parentPropName)];
            print('------ Parent Data $parentPropName $parentData');
            var childData = parentData[widget.property.lowerFirst(childPropName)];
            print('------ Child Data $childPropName $childData' );
            if(childData == null){
              var parentEntity = EntityService.getEntity(parentPropName);
              var childProperty = parentEntity.properties.firstWhere((e) => e.dbName.toLowerCase() == childPropName.toLowerCase());
              if(childProperty is PropertyOtherEntityMany){
                var result = await EntityService().getData(EntityService.getEntity(childProperty.entityName),activeOnly:true,forceFromServer:true, filter: [EntityFilter(operatorId: "E",propertyName: parentEntity.dbName +"Id", value: parentData["id"] )]);
                if(result.success && result.data.length > 0 && childProperty.entityName.toLowerCase() != widget.entity.dbName.toLowerCase()){
                  var childEntity =EntityService.getEntity(childProperty.entityName);
                  var childRelationProperty = childEntity.properties.firstWhere((e) => e.propertyType == PropertyTypes.Dropdown && (e as PropertyDropdown).entityName == widget.entity);
                  result.data = result.data.map((e) => e[childRelationProperty.dbName.substring(0,childRelationProperty.dbName.indexOf("Id"))]).toList();                                    
                }
                return result;
              }
            }
          }else{
            // TODO handel this case
            return OperationResult(success: false);
          }
        }else{
          return await EntityService().getData(entity,pageIndex: 0,activeOnly:true,filter: [EntityFilter(operatorId: 'E', value: widget.data[widget.property.filterValueField],property: entity.properties.firstWhere((element) => element.dbName == widget.property.filterField))]);
        }
      }
    }
    if(widget.urlFilterData == null) return OperationResult(success: false);
    var url = widget.url;
    for(var p in widget.urlFilterData.keys){
      print('p=$p     value= ${widget.urlFilterData[p].toString()}');
      url = url.replaceAll('{$p}', widget.urlFilterData[p].toString());
    }
    return await ApiService.httpGetDynamic(url);    
    }catch(err){
      return OperationResult(success: false,message: err.toString());
    }
  }

  Future<void> onRefresh() async{
    pageIndex=0;
    var result = await getList(forceFromServer: true);
    //print(result.data);
    if(result.success){
      data = result.data.where((d)=> !widget.property.isFilter || widget.data[widget.property.filterValueField] == d[widget.property.filterField]).toList();
      setState(() {});   
    } else result.showMessage(context);
  }

  onLocalSearch(String searchText){
    try{
      if(searchText.isEmpty)
        dataFilterd = null;
      else
        dataFilterd =data.where((d)=> entity.gridProperties.any((p)=> d[p.dbName].toString().contains(searchText) )).toList();      
      setState(() {});
    }catch(err){
      print('XXXXXXXXXXX Local Search Error: $err');
    }
  }

  onServerSearch(String searchText)async{
    try{
    if(searchText.isEmpty)
      dataFilterd = null;
    else
    {
      var searchProp = entity.gridProperties.where((e) => e.isDisplayField).first;
      //print('server search ${searchProp.dbName} $searchText');
      var result = await EntityService().getData(entity,pageIndex: 0,activeOnly:true,filter: [EntityFilter(operatorId: 'C', value: searchText,property: searchProp)]);
      if(!result.success){
        AppNavigator.showMessage(context, 'Server Error');
      }else
      dataFilterd = result.data;

      //print('server search $dataFilterd');
    }    
    setState(() {});
    }catch(err){
      print('XXXXXXXXXXX Server Search Error: $err');
    }
  }


  @override
  Widget build(BuildContext context) {
    return SizedBox(width: 300,child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
        SearchBox(onChange:(v) => onLocalSearch(v),onSubmit: (v)=> onServerSearch(v)),
        Expanded(child: data == null ? Center(child: CircularProgressIndicator(valueColor:AlwaysStoppedAnimation<Color>(ThemeService.mainColor))) : 
        RefreshIndicator(
        onRefresh: onRefresh,
        child:NotificationListener<ScrollNotification>(
                onNotification: (scrollNotification){
                  if (scrollNotification is ScrollStartNotification) {
                    //print('scroll start');
                    FocusScope.of(context).requestFocus(new FocusNode());
                  }
                  return false;
                },
        child:ListView(
          controller: scrollController,
          children: (dataFilterd ?? data).map<Widget>((d)=>EntityListTileComponent(entity,d,onClick: widget.onClick)).toList()
        )))),
      ],));
  }
}