part of tecfy_core_client_package;

class EntityListPage extends StatefulWidget {
  final String entity;
  EntityListPage(this.entity);
  @override
  _EntityListPageState createState() => _EntityListPageState();
}

class _EntityListPageState extends State<EntityListPage> with TickerProviderStateMixin{

  EntityListController _controller;

  @override
  void initState() {
    _controller = EntityListController(this,widget.entity);
    super.initState();
  }

  onActionClicked(EntityAction action, Map data)async{
   Map form = Map<String,dynamic>();
   await AppNavigator.showAppDialog(context, (dialogContext)=> 
   Container(
     width: 300,
     height: 500,
     child: Card(
      child: Column(
        children: <Widget>[
         Chip(label:Text(action.name)),
          Column(
            children: action.actionFormProp.map((p)=> PorpertyComponent(property: p, onChange: (v)=>form[p.dbName] =v)).toList(),
          ),
          AppButtonPost(label: action.name, onPressed: ()async{
            Navigator.pop(dialogContext);
          })
        ],
      ),
    )));
    // print(result);
    // print(form);
    for(var prop in action.actionFormProp){
      if(prop.isRequired && form[prop.dbName] == null)
      {
        AppNavigator.showMessage(context, 'Property required : ${prop.name}');
        return;
      }
    }
    _controller.setLoading();
    form['id'] = data['id'];
    var postResult= await ApiService.httpPost('Entity/${_controller.entity.dbName}/action/${action.dbName}', form,true);
    _controller.setResume();
    _controller.showMessage(postResult.success? 'Done' : 'Unknown Error!');
  }

  onOpenFilter()async{
    var result = await AppNavigator.navigateTo(context, ()=>EntityFilterPage(_controller.entity.name,_controller.entity.properties ,_controller.filters),animationType: AnimationTypes.slideLeft) ;
    bool canceled =false;
    if(result == null) canceled =true;
    _controller.filters = (result as List<EntityFilter>).where((f)=>f.value != null && f.property != null && f.operatorId != null).toList();

    if(!canceled){
      _controller.setLoading();
      _controller.loadData();
    }
  }

  @override
  Widget build(BuildContext context) {
    return 
    BasePage(context,_controller.entity.name, 
        appBarActions: LayoutService.isMobile(context) ? 
          [
            IconButton(icon:Icon(Icons.filter_list,color: _controller.filters.length > 0 ? Colors.redAccent: Colors.white), onPressed: onOpenFilter) 
          ] : 
          [
            FlatButton(child: Text(AppLocale.translate(context, 'Add'),style: TextStyle(color: Colors.white)), onPressed: _controller.onAdd),
            FlatButton(child: Text(AppLocale.translate(context,'Refresh'),style: TextStyle(color: Colors.white)), onPressed: _controller.onRefresh),
          ],
        floatingActionButtonWidget:  LayoutService.isMobile(context) && _controller.entity.permission.canAdd ? 
                  FloatingActionButton(backgroundColor: ThemeService.mainColor,onPressed: _controller.onAdd,child: Icon(Icons.add)):null,

        child:_controller.data == null ? Center(child:CircularProgressIndicator(valueColor:AlwaysStoppedAnimation<Color>(ThemeService.mainColor))) :
                      LayoutService.bindWidget(context, mobileWidget: getMobileWidget(), descktopWidget:getDesctopWidget())
        );
  }
          
  Widget getDesctopWidget(){
      return Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
        Card(
          margin: EdgeInsets.all(10),
          child: Padding(padding: EdgeInsets.all(10),
          child:Text('Filter')
          ),
        ),
        Expanded(child: 
        Scrollbar(
          child:SingleChildScrollView(
          child: Card(
            margin: EdgeInsets.all(10),
            child: Padding(padding: EdgeInsets.all(10),child:
          Table(
            columnWidths:  {0:FlexColumnWidth(1),1:FlexColumnWidth(3)},
            children: _controller.data.map<TableRow>((d)=>
              TableRow(
                children: _controller.entity.properties.where((e)=>e.isGrid && (e.language == Language.notSelected || e.language == LocalConfigService.language)).map<TableCell>((p)=> getCell(p, d)).toList()
                ..add(actionCell(d)),
              )
            ).toList()
            ..insert(0, TableRow(
              children: _controller.entity.properties.where((e)=>e.isGrid  && (e.language == Language.notSelected || e.language == LocalConfigService.language)).map<TableCell>((p)=>getHeader(p)).toList()
              ..add(TableCell(child:Padding(padding:EdgeInsets.only(top:10) ,child: Text(AppLocale.translate(context,'Action'), style: ThemeService.titleStyle)))),
            ))
          ))),
        ))),
       _controller.hasPageing ? SizedBox(
          height: 50,
          child:ListView.builder(itemCount: (_controller.pageIndexMax+2),
                         scrollDirection: Axis.horizontal,
                         itemBuilder: (_context,i)=> i == _controller.pageIndexMax+1 ? 
                            (_controller.hasNextPage ? FlatButton(child: Text(AppLocale.translate(context, 'Next Page')),onPressed: _controller.gotoMextPageIndex) : Container()):
                              IconButton(padding: EdgeInsets.all(0), icon:Text((i+1).toString(), style: TextStyle(color: i == _controller.pageIndex ? ThemeService.mainColor : ThemeService.secondryColor),),onPressed: ()=>_controller.gotoPageIndex(i)) 
          )): Container()
        
        ]);

    }

  TableCell actionCell(Map data){

    return TableCell(
      child: Row(children: <Widget>[
        IconButton(padding: EdgeInsets.all(0), icon: Icon(Icons.edit,color: ThemeService.mainColor),onPressed: ()=>_controller.onUpdate(data)),
        IconButton(padding: EdgeInsets.all(0), icon: Icon(Icons.delete,color: ThemeService.dangerColor),onPressed: ()=>_controller.onDelete(data)),
      ])
      );
  }

  TableCell getHeader(Property prop){
    return TableCell(child: MaterialButton(padding: EdgeInsets.all(0), child: Align(alignment: LocalConfigService.isArabic ?  Alignment.centerRight :  Alignment.centerLeft,child: 
     Text(prop.name, textAlign: TextAlign.start, style: ThemeService.titleStyle.copyWith(color: prop.dbName == _controller.sortProp ? ThemeService.mainColor : Colors.black))),      
     onPressed: ()=>_controller.sortBy(prop)));
  }

  TableCell getCell(Property prop,data,{totals:false}){
    if(prop.isImage)
    {
      if(data[prop.dbName+'Id'] == null || data[prop.dbName+'Id'].toString().isEmpty)
      {
        return TableCell(child:Container());
      }
      return TableCell(child: ImageService.image(data[prop.dbName+'Id'],height: 70,width:70));
    }

    var val = data[prop.dbName];    
    if(totals){
      if(val == null || val == 0 || val == '' || val == '0') return TableCell(child:Container());
    }
    else{
      if(val == null) return TableCell(child:Container());
    }

    Widget widgetValue;
    switch(prop.valueType){
      case ValueTypes.Text:
      widgetValue = Text(val.toString(), style: totals? ThemeService.titleStyle : TextStyle(fontSize: 12));
      break;
      case ValueTypes.Number:
      //widgetValue =Text(val.toString(),style: totals? ThemeService.titleStyle :TextStyle(fontSize: 12));
      if(prop.propertyType == PropertyTypes.DateTime)
        widgetValue = Text( DateTime.fromMillisecondsSinceEpoch(val).toString() ,style: totals? ThemeService.titleStyle :TextStyle(fontSize: 10));
      else
        widgetValue = prop.getDisplayWidget(data);
      break;
      case ValueTypes.DateTime:
      if(prop.propertyType == PropertyTypes.DateTime)
        widgetValue = Text( val.toString().replaceFirst('T', ' ').substring(0,16),style: totals? ThemeService.titleStyle :TextStyle(fontSize: 10));
      else
        widgetValue = Text(val.toString().length > 10 ? val.toString().substring(0,10) : val.toString(),style: totals? ThemeService.titleStyle :TextStyle(fontSize: 10));
      break;
      case ValueTypes.Boolean:
      widgetValue = Align(alignment: LocalConfigService.isArabic ? Alignment.centerRight : Alignment.centerLeft,child: prop.getDisplayWidget(data));// Text( val.toString(),style: totals? ThemeService.titleStyle :TextStyle(fontSize: 10));
      break;
      default:
      widgetValue = Text('N/A [${prop.valueType}]');
    }

    return TableCell(child: Padding(padding: EdgeInsets.only(top: 10),child:widgetValue));
  }

  Widget getMobileWidget(){
    return  RefreshIndicator(
        onRefresh: _controller.loadFromServer,
        child: NotificationListener<ScrollNotification>(
                onNotification: (scrollNotification){
                  if (scrollNotification is ScrollStartNotification) {
                    print('scroll start');
                    FocusScope.of(context).requestFocus(new FocusNode());
                  }
                  return false;
                },
        child:ListView(
          controller: _controller.scrollController,
          children: _controller.data.map<Widget>((d)=>
        Slidable(
        // delegate: SlidableDrawerDelegate(),
        actionPane: SlidableDrawerActionPane(),
        actionExtentRatio: 0.25,
        child: 
        Container(
          color: Colors.white,
          child:EntityListTileComponent(_controller.entity,d,onClick: (d)=>_controller.onUpdate(d))),

            actions: _controller.entity.actions?.where((a)=> d[a.dbName+'When'])?.map((a)=>
              IconSlideAction(
                caption: a.name,
                color: ThemeService.mainColor,
                icon: Icons.done,
                onTap: () => onActionClicked(a,d) //_showSnackBar('Archive'),
              ),
            )?.toList() ?? [],
            // <Widget>[
            //   IconSlideAction(
            //     caption: 'Archive',
            //     color: Colors.blue,
            //     icon: Icons.archive,
            //     onTap: () => _showSnackBar('Archive'),
            //   ),
            //   IconSlideAction(
            //     caption: 'Share',
            //     color: Colors.indigo,
            //     icon: Icons.share,
            //     onTap: () => _showSnackBar('Share'),
            //   ),
            // ],
            secondaryActions: !_controller.entity.permission.canDelete ? [] : <Widget>[
              // IconSlideAction(
              //   caption: 'More',
              //   color: Colors.black45,
              //   icon: Icons.more_horiz,
              //   onTap: () => _showSnackBar('More'),
              // ),
              IconSlideAction(
                caption: AppLocale.translate(context,'Delete'),
                color: Colors.red,
                icon: Icons.delete,
                onTap: () => _controller.onDelete(d),
              ),
            ],
              )
      
            ).toList()          
            ..insert(0, SizedBox(height:_controller.size1))
            ..insert(_controller.data.length > 1 ? 2 : 1, SizedBox(height:_controller.data.length > 1 ? _controller.size2 : 0))
            ..insert(_controller.data.length > 2 ? 4 : 1, SizedBox(height: _controller.data.length > 2 ? _controller.size3 : 0))
            ..insert(_controller.data.length > 3 ? 6 : 1, SizedBox(height:_controller.data.length > 3 ? _controller.size4 : 0))
            ..add(_controller.hasNextPage ? FlatButton(child:Text('Load More'),onPressed: _controller.gotoMextPageIndex) : Container())
            ),
            ));
      }

}